from leapp.libraries.stdlib import api, run
from leapp.models import InstalledTargetKernelVersion
import os

#get kernel-core version from software repository
def _get_newest_kernel_version():
    target_version= os.environ['LEAPP_UPGRADE_PATH_TARGET_RELEASE']
    
    if os.path.exists('/etc/yum.repos.d/leapp-upgrade-repo-oc.repo'):
        command = ["dnf", "--showduplicates", "--disablerepo=*","--releasever={}".format(target_version), "--enablerepo=baseos-rpms,appstream-rpms", "list", "kernel-core"]
        output = run(command, split=True)
        # the output is a list of lines, we need to filter the lines that contain 'kernel-core'
        filtered_output = [line for line in output['stdout'] if 'kernel-core' in line]
        # get the last line from the filered output,which is the newest kernel-core version
        number_of_kernelcore = len(filtered_output)
        original_version = filtered_output[number_of_kernelcore-1] if filtered_output else None

        basearch_command = ['uname','-m']
        basearch = run(basearch_command, split=True)['stdout']
        if 'aarch64' in basearch:
            basearch = 'aarch64'
        elif 'x86_64' in basearch:
            basearch = 'x86_64'

        if 'oc9' in original_version:
            # original_version contains three parts, the first part is the package name and arch, the second part is the verison-release,the third part is the rpeository name
            # the output of 'uname -r' is 'version-release.arch'
            kernel_version= original_version.split()[1] + '.{}'.format(basearch)
            api.current_logger().info('kernel_version is {}'.format(kernel_version))
            return kernel_version
def process():
    version = _get_newest_kernel_version()
    if version:
        print(version)
        api.produce(InstalledTargetKernelVersion(version=version))
    else:
        api.current_logger().error('Cannot detect any kernel-core RPM')
