from leapp.libraries.common import grub
from leapp.libraries.common.config import architecture
from leapp.libraries.stdlib import api, CalledProcessError, run
from leapp.models import DefaultGrubInfo, FirmwareFacts
import os,sys,subprocess

GRUB_CFG_PATH = '/boot/grub2/grub.cfg'


def _is_grub_cfg_bls():
    # We do not want to use info from the GrubCfgBios model as it may not be valid
    # after the RPM upgrade transaction.

    with open(GRUB_CFG_PATH, 'r') as fo:
        grub_cfg = fo.read()
    if 'insmod blscfg' in grub_cfg:
        return True
    return False

def GetDiskInfo(diststr):
    ''' Get system disk info '''
    dev_name = ""
    part_name = ""
    length = len(diststr)
    for c in range(length-1, -1, -1):
        if not diststr[c].isdigit():
            if diststr.find('nvme') != -1:
                dev_name = diststr[0:c]
                part_num = diststr[c+1:length]
            else:
                dev_name = diststr[0:c+1]
                part_num = diststr[c+1:length]
                break
    return dev_name,part_num

def SetEfibootmgr():
    print("[INFO]: Current system is uefi, add boot option to boot manager.")
    disk_name = ""
    disk_name = subprocess.check_output('mount | grep /boot/efi | awk \'{print $1}\'', shell=True)
    disk_name = str(disk_name, 'utf-8')
    disk_name = disk_name.split('\n')[0]
    dev_name,part_num = GetDiskInfo(disk_name)
    if dev_name == "" or part_num == "":
        print("[Fail]: Parse /boot/efi disk info failed, update boot loader failed.")
        sys.exit(1)
    cmd = ""
    
    if architecture.matches_architecture(architecture.ARCH_ARM64):
        cmd = 'efibootmgr -c -d ' + dev_name + ' -p ' + part_num + ' -l "/EFI/opencloudos/shimx64.efi" -L "Migrate To OpenCloudOS 9"'
    efi_rst = subprocess.run(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
    if efi_rst.returncode != 0:
        print("[Fail]: Use efibootmgr update boot loader failed.")
        sys.exit(1)

def process():
    if not architecture.matches_architecture(architecture.ARCH_ARM64):
        return
    if os.path.isdir('/sys/firmware/efi'):
        subprocess.run('grub2-mkconfig -o /boot/efi/EFI/opencloudos/grub.cfg', shell=True)
        SetEfibootmgr()
    return
    default_grub_msg = next(api.consume(DefaultGrubInfo), None)
    ff = next(api.consume(FirmwareFacts), None)
    if ff and ff.ppc64le_opal:
        return
    if (
        not _is_grub_cfg_bls() and
        grub.is_blscfg_enabled_in_defaultgrub(default_grub_msg)
    ):
        try:
            run(['grub2-mkconfig', '-o', GRUB_CFG_PATH])
        except CalledProcessError as e:
            api.current_logger().error(
                'Command grub2-mkconfig -o {} failed: {}'.format(GRUB_CFG_PATH, e)
            )
